@@ -104,6 +104,7 @@ char *cdn_result_codes[] = {
     "Call failed due to lack of appropriate facilities being available (permanent condition)",
     "Invalid destination",
     "Call failed due to no carrier detected",
+    "Call failed due to detection of a busy signal",
     "Call failed due to lack of a dial tone",
     "Call was no established within time allotted by LAC",
     "Call was connected but no appropriate framing was detect"
@@ -129,7 +130,7 @@ struct unaligned_u16 {
 } __attribute__((packed));
 
 /*
- * t, c, data, and datalen may be assumed to be defined for all avp's
+ * t, c, data, and datalen may be assumed to be defined for all AVP's
  */
 
 int message_type_avp (struct tunnel *t, struct call *c, void *data,
@@ -166,7 +167,7 @@ int message_type_avp (struct tunnel *t, struct call *c, void *data,
     if (t->sanity)
     {
         /*
-         * Look ou our state for each message and make sure everything
+         * Look out our state for each message and make sure everything
          * make sense...
          */
         if ((c != t->self) && (c->msgtype < Hello))
@@ -380,8 +381,8 @@ int ignore_avp (struct tunnel *t, struct call *c, void *data, int datalen)
      * The spec says we have to accept authentication information
      * even if we just ignore it, so that's exactly what
      * we're going to do at this point.  Proxy authentication is such
-     * a rediculous security threat anyway except from local
-     * controled machines.
+     * a ridiculous security threat anyway except from local
+     * controlled machines.
      *
      * FIXME: I need to handle proxy authentication as an option.
      * One option is to simply change the options we pass to pppd.
@@ -435,7 +436,7 @@ int result_code_avp (struct tunnel *t, struct call *c, void *data,
                      int datalen)
 {
     /*
-     * Find out what version of l2tp the other side is using.
+     * Find out what version of L2TP the other side is using.
      * I'm not sure what we're supposed to do with this but whatever..
      */
 
@@ -538,7 +539,7 @@ int protocol_version_avp (struct tunnel *t, struct call *c, void *data,
                           int datalen)
 {
     /*
-     * Find out what version of l2tp the other side is using.
+     * Find out what version of L2TP the other side is using.
      * I'm not sure what we're supposed to do with this but whatever..
      */
 
@@ -1477,7 +1478,7 @@ int rx_speed_avp (struct tunnel *t, struct call *c, void *data, int datalen)
 int tx_speed_avp (struct tunnel *t, struct call *c, void *data, int datalen)
 {
     /*
-     * What is the tranmsit baud rate of the call?
+     * What is the transmit baud rate of the call?
      */
     struct unaligned_u16 *raw = data;
 
@@ -1633,11 +1634,14 @@ int handle_avps (struct buffer *buf, struct tunnel *t, struct call *c)
     int hidlen = 0;
     char *data = buf->start + sizeof (struct control_hdr);
     avp = (struct avp_hdr *) data;
+    /* I had to comment out the following since Valgrind tells me it leaks like my bathroom faucet
     if (gconfig.debug_avp)
         l2tp_log (LOG_DEBUG, "%s: handling avp's for tunnel %d, call %d\n",
              __FUNCTION__, t->ourtid, c->ourcid);
+    */
     while (len > 0)
     {
+        hidlen = 0;
         /* Go ahead and byte-swap the header */
         swaps (avp, sizeof (struct avp_hdr));
         if (avp->attr > AVP_MAX)
@@ -1658,7 +1662,7 @@ int handle_avps (struct buffer *buf, struct tunnel *t, struct call *c)
             {
                 if (DEBUG)
                     l2tp_log (LOG_WARNING,
-                         "%s:  don't know how to handle atribute %d.\n",
+                         "%s:  don't know how to handle attribute %d.\n",
                          __FUNCTION__, avp->attr);
                 goto next;
             }
@@ -1706,7 +1710,7 @@ int handle_avps (struct buffer *buf, struct tunnel *t, struct call *c)
             l2tp_log (LOG_DEBUG, "%s: Hidden bit set on AVP.\n", __FUNCTION__);
 #endif
             /* We want to rewrite the AVP as an unhidden AVP
-               and then pass it along as normal.  Remeber how
+               and then pass it along as normal.  Remember how
                long the AVP was in the first place though! */
             hidlen = avp->length;
             if (decrypt_avp (data, t))
@@ -1769,23 +1773,28 @@ int handle_avps (struct buffer *buf, struct tunnel *t, struct call *c)
             else
             {
                 if (DEBUG)
-                    l2tp_log (LOG_WARNING, "%s:  no handler for atribute %d (%s).\n",
+                    l2tp_log (LOG_WARNING, "%s:  no handler for attribute %d (%s).\n",
                          __FUNCTION__, avp->attr,
                          avps[avp->attr].description);
             }
         }
       next:
-        if (hidlen)
+        if (hidlen && ALENGTH(hidlen))
         {
             /* Skip over the complete length of the hidden AVP */
             len -= ALENGTH (hidlen);
             data += ALENGTH (hidlen);
         }
-        else
+        else if (ALENGTH(avp->length))
         {
             len -= ALENGTH (avp->length);
             data += ALENGTH (avp->length);      /* Next AVP, please */
         }
+        else
+        {
+            l2tp_log (LOG_WARNING, "%s: broken avp->length zero %d\n", __FUNCTION__,avp->length);
+            break;
+        }
         avp = (struct avp_hdr *) data;
         firstavp = 0;
     }
